unit QCalendar;

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
(*               Copyright (C) 2003 by Luca Tomat                  *)
(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
(* LICENSE AGREEMENT (Modified BSD License)                        *)
(*                                                                 *)
(* Redistribution and use in source and binary forms, with or      *)
(* without modification, are permitted provided that the following *)
(* conditions are met:                                             *)
(*                                                                 *)
(* 1) Redistributions of source code must retain the above         *)
(*    copyright notice, this list of conditions and                *)
(*    the following disclaimer.                                    *)
(*                                                                 *)
(* 2) Redistributions in binary form must reproduce the above      *)
(*    copyright notice, this list of conditions and the following  *)
(*    disclaimer in the documentation and/or other materials       *)
(*    provided with the distribution.                              *)
(*                                                                 *)
(* 3) The name of the author may not be used to endorse or promote *)
(*    products derived from this software without specific prior   *)
(*    written permission.                                          *)
(*                                                                 *)
(* THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY         *)
(* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,   *)
(* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR       *)
(* A PARTICULAR PURPOSE ARE DISCLAIMED.                            *)
(* IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,          *)
(* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,                       *)
(* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        *)
(* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, *)
(* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY *)
(* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     *)
(* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY  *)
(* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED                *)
(* OF THE POSSIBILITY OF SUCH DAMAGE.                              *)
(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
(*                          History:                               *)
(*                                                                 *)
(* Version 0.1 - May 24, 2003 [Luca T.]: First release. Works      *)
(*               without errors/warnings on Kylix 3.               *)
(*                                                                 *)
(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
(*                          Contacts:                              *)
(*                                                                 *)
(*                    ---Original Author---                        *)
(*                                                                 *)
(* Name & Lastname: Luca Tomat                                     *)
(* Email Address  : luca@lt-software.com, luca-t@libero.it         *)
(* Website        : http://www.lt-software.com                     *)
(*                                                                 *)
(* Please, don't send me Word/Excel or other Microsoft based       *)
(* documents. Use standard emails, text files, rtf files or        *)
(* OpenOffice (http://www.openoffice.org) files instead.           *)
(*                                                                 *)
(* If you have suggestions, comments or improvements for this      *)
(* component don't hesitate... contact me!                         *)
(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)

interface

uses
  SysUtils, Classes, QControls, QStdCtrls, QExtCtrls, QGraphics, DateUtils,
  QComCtrls, Math, QForms, QGrids, Types, QButtons, Qt;

type
  TDoW = (dowMonday,dowTuesday,dowWednesday,dowThursday,
          dowFriday,dowSaturday,dowSunday);
  TDType = (dtNormal,dtPrevMonth,dtNextMonth,dtCurrent);
  TMonthGridElement = record
                        Day: TDatetime;
                        DType: TDType;
                        Selected: Boolean;
                      end;
  TCalendar = class(TFrameControl)
  private
    { Private declarations }
    FFDoW: TDoW;
    FEnabled: boolean;
    FDate: TDateTime;
    FHeaderColor: TColor;
    FTodayLabel: TLabel;
    FHeaderPanel: TPanel;
    FBodyPanel: TPanel;
    FGrid: TDrawGrid;
    FMonth: TSpeedButton;
    FYear: TEdit;
    FTodayLabelCaption: String;
    FCreated :boolean;
    FoldMonthFontChangeProc: TNotifyEvent;
    FoldYearFontChangeProc: TNotifyEvent;
    FoldYearColor: TColor;
    FoldYear: String;
    FOnDateChanged: TNotifyEvent;
    FOnMouseMove: TMouseMoveEvent;
    FMonthGrid: array[1..7,1..6] of TMonthGridElement;
    FDayNamesFont: TFont;
    FTodayFont: TFont;
    FMonthDayFont: TFont;
    FOtherDayFont: TFont;
    FMinHeaderWidth: Integer;
    FMinBodyWidth: Integer;
    FMinHeaderHeight: Integer;
    FMinBodyHeight: Integer;
    FMinTodayLabelHeight: Integer;
    procedure MonthFontChange(Sender :TObject);
    procedure YearFontChange(Sender :TObject);
    function ReadLabelFont :TFont;
    function ReadLabelColor: TColor;
    procedure SetLabelColor(const Value: TColor);
    function ReadLabelCaption: String;
    procedure SetLabelCaption(const Value: String);
    function ReadHeaderColor: TColor;
    procedure SetHeaderColor(const Value: TColor);
    function TextWidth(Font :TFont; Text :String) :integer;
    function TextHeight(Font :TFont; Text :String) :integer;
    procedure AutoSizeAndPlace;
    procedure HeaderAutoSizeAndPlace;
    function ReadMonthFont: TFont;
    function ReadYearFont: TFont;
    function ReadCalColor: TColor;
    procedure SetCalColor(const Value: TColor);
    procedure AutoSizeCalendar;
    procedure CalendarDrawCell(Sender: TObject; ACol, ARow: Integer;
                               Rect: TRect; State: TGridDrawState);
    procedure SetFDoW(const Value: TDoW);
    procedure YearOnExit(Sender: TObject);
    procedure YearOnEnter(Sender: TObject);
    procedure SetLabelFont(const Value: TFont);
    procedure SetYearFont(const Value: TFont);
    procedure SetMonthFont(const Value: TFont);
    function MonthName(Dt: TDateTime): String;
    function ReadLabelAlignment: TAlignment;
    procedure SetLabelAlignment(const Value: TAlignment);
    procedure DateChanged;
    procedure ShowMonthMenu(Sender: TObject);
    procedure MonthButtonClick(Sender: TObject);
    procedure YearOnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure CloseMonthMenu(Sender: TObject);
    procedure FillMonthGrid;
    function ReadDayNamesFont: TFont;
    procedure SetDayNamesFont(const Value: TFont);
    function ReadTodayFont: TFont;
    procedure SetTodayFont(const Value: TFont);
    function ReadMonthDayFont: TFont;
    procedure SetMonthDayFont(const Value: TFont);
    function ReadOtherDayFont: TFont;
    procedure SetOtherDayFont(const Value: TFont);
    procedure CalendarClick(Sender: TObject);
    procedure SetDate(const Value: TDateTime);
    procedure YearOnMouseMove(Sender: TObject; Shift: TShiftState;
                              X, Y: Integer);
    procedure MonthOnMouseMove(Sender: TObject; Shift: TShiftState;
                               X, Y: Integer);
    procedure HeaderOnMouseMove(Sender: TObject; Shift: TShiftState;
                                X, Y: Integer);
    procedure BodyOnMouseMove(Sender: TObject; Shift: TShiftState;
                              X, Y: Integer);
    procedure GridOnMouseMove(Sender: TObject; Shift: TShiftState;
                              X, Y: Integer);
    procedure TodayLabelOnMouseMove(Sender: TObject; Shift: TShiftState;
                                    X, Y: Integer);
  protected
    { Protected declarations }
    procedure Resize; override;
    procedure SetEnabled(const Value: Boolean); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Repaint; override;
  published
    { Published declarations }
    property BorderStyle;
    property Date: TDateTime read FDate write SetDate;
    property FirstDayOfWeek: TDoW read FFDoW write SetFDoW default dowMonday;
    property DayNamesFont: TFont read ReadDayNamesFont write SetDayNamesFont;
    property TodayFont: TFont read ReadTodayFont write SetTodayFont;
    property MonthDayFont: TFont read ReadMonthDayFont write SetMonthDayFont;
    property OtherDayFont: TFont read ReadOtherDayFont write SetOtherDayFont;
    property CalendarColor: TColor read ReadCalColor  write SetCalColor;
    property HeaderColor: TColor read ReadHeaderColor write SetHeaderColor;
    property MonthFont: TFont read ReadMonthFont write SetMonthFont;
    property YearFont: TFont read ReadYearFont write SetYearFont;
    property TodayLabelFont: TFont read ReadLabelFont write SetLabelFont;
    property TodayLabelColor: TColor read ReadLabelColor write SetLabelColor;
    property TodayLabelCaption: String read ReadLabelCaption
                                       write SetLabelCaption;
    property TodayLabelAlignment: TAlignment read ReadLabelAlignment
                                             write SetLabelAlignment;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property OnResize;
    property OnDateChanged: TNotifyEvent read FOnDateChanged
                                         write FOnDateChanged;
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    //property OnMouseEnter;
    //property OnMouseLeave;
    property TabStop;
    property TabOrder;
  end;

procedure Register;

implementation

{$R QCalendar.dcr}

procedure Register;
begin
  RegisterComponents('LTSoftware', [TCalendar]);
end;

{ TCalendar }

procedure TCalendar.AutoSizeAndPlace;
const
 MinSpaceX = 5;
 MinSpaceY = 5;
begin
  HeaderAutoSizeAndPlace;
  AutoSizeCalendar;
  FMinTodayLabelHeight := TextHeight(FTodayLabel.Font,FTodayLabel.Caption);
  Constraints.MinHeight := FMinBodyHeight+FMinHeaderHeight+
                           FMinTodayLabelHeight+MinSpaceY*2;
  if Height < Constraints.MinHeight then Height := Constraints.MinHeight;
  Constraints.MinWidth := max(FMinBodyWidth,FMinHeaderWidth)+MinSpaceX;
  if Width < Constraints.MinWidth then Width := Constraints.MinWidth;
end;

procedure TCalendar.AutoSizeCalendar;
var
  maxCellHeight, maxCellWidth: Integer;
  i: Byte;
  DayName: String;
const
  minSpaceX = 6;
  minSpaceY = 2;
begin
  maxCellHeight := 0;
  maxCellWidth := 0;
  //Find Max width and height of the text in the cells
  //DayNames
  for i := 1 to 7 do
   begin
    DayName := UpperCase(copy(ShortDayNames[i],1,1)) +
               copy(ShortDayNames[i],2,length(ShortDayNames[i])-1);
    maxCellHeight := max(TextHeight(FDayNamesFont,DayName),maxCellHeight);
    maxCellWidth := max(TextWidth(FDayNamesFont,DayName),maxCellWidth);
   end;
  //Today
  maxCellHeight := max(TextHeight(FTodayFont,'00'),maxCellHeight);
  maxCellWidth := max(TextWidth(FTodayFont,'00'),maxCellWidth);
  //MonthDays
  maxCellHeight := max(TextHeight(FMonthDayFont,'00'),maxCellHeight);
  maxCellWidth := max(TextWidth(FMonthDayFont,'00'),maxCellWidth);
  //OtherDays
  maxCellHeight := max(TextHeight(FOtherDayFont,'00'),maxCellHeight);
  maxCellWidth := max(TextWidth(FOtherDayFont,'00'),maxCellWidth);
  maxCellHeight := maxCellHeight+minSpaceY;
  maxCellWidth :=  maxCellWidth+minSpaceX;
  for i := 0 to 6 do FGrid.ColWidths[i] := maxCellWidth;
  for i := 0 to 6 do FGrid.RowHeights[i] := maxCellHeight;
  FGrid.Width := maxCellWidth*7;
  FGrid.Height := maxCellHeight*7;
  FGrid.Left := (FGrid.Parent.Width-FGrid.Width) div 2;
  FGrid.Top := (FGrid.Parent.Height-FGrid.Height) div 2;
  FMinBodyWidth := FGrid.Width;
  FMinBodyHeight := FGrid.Height;
  FGrid.Invalidate;
end;

procedure TCalendar.BodyOnMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  If Assigned(FOnMouseMove) then
   begin
    FOnMouseMove(Sender,Shift,FBodyPanel.Left+X,FBodyPanel.Top+Y);
   end;
end;

procedure TCalendar.CalendarClick(Sender: TObject);
var
  MyRow,MyCol: Integer;
begin
  with FGrid.ScreenToClient(Mouse.CursorPos) do
   FGrid.MouseToCell(X,Y,MyCol,MyRow);
  if (MyCol<0) or (MyCol>6) or (MyRow<1) or (MyRow>6) then exit;
  if DateOf(FDate) <> DateOf(FMonthGrid[MyCol+1,MyRow].Day) then
   begin
    FDate := FMonthGrid[MyCol+1,MyRow].Day;
    DateChanged;
   end;
end;

procedure TCalendar.CalendarDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  DayName: string;
  WeekDay: Byte;
  isSelected: Boolean;
begin
  isSelected := False;
  if ARow = 0 then
   begin
    //First Row (DayNames Row)
    WeekDay := 1;
    case FFDow of
     dowSunday: WeekDay := 1;
     dowMonday: WeekDay := 2;
     dowTuesday: WeekDay := 3;
     dowWednesday: WeekDay := 4;
     dowThursday: WeekDay := 5;
     dowFriday: WeekDay := 6;
     dowSaturday: WeekDay := 7;
    end;
    DayName := ShortDayNames[((ACol+WeekDay-1) mod 7) + 1];
    DayName := UpperCase(copy(DayName,1,1)) + copy(DayName,2,length(DayName)-1);
    FGrid.Canvas.Font.Assign(FDayNamesFont);
   end
   else begin
    //Other Rows
    DayName := IntToStr(DayOf(FMonthGrid[ACol+1,ARow].Day));
    case FMonthGrid[ACol+1,ARow].DType of
     dtCurrent: FGrid.Canvas.Font.Assign(FTodayFont);
     dtPrevMonth,dtNextMonth: FGrid.Canvas.Font.Assign(FOtherDayFont);
     dtNormal: FGrid.Canvas.Font.Assign(FMonthDayFont);
    end;
    if FMonthGrid[ACol+1,ARow].Selected then isSelected := True;
   end;
   with FGrid.Canvas do
    begin
     Brush.Color := FGrid.Color;
     Brush.Style := bsSolid;
     FillRect(Rect);
     if ARow = 0 then
      begin
       Pen.Style := psSolid;
       Pen.Width := 1;
       if ACol = 0 then MoveTo(Rect.Left+2,Rect.Bottom-1)
                   else MoveTo(Rect.Left,Rect.Bottom-1);
       if ACol = 6 then LineTo(Rect.Right-2,Rect.Bottom-1)
                   else LineTo(Rect.Right,Rect.Bottom-1);
      end;
     if isSelected then
      begin
       Pen.Style := psDot;
       Pen.Width := 1;
       Font.Style := Font.Style + [fsBold,fsUnderline]; //Why isn't it bold???
       Rectangle(Rect);
      end;
     TextRect(Rect,Rect.Left,Rect.Top,DayName,ord(AlignmentFlags_AlignCenter));
    end;
end;

procedure TCalendar.CloseMonthMenu(Sender: TObject);
begin
  (Sender as TForm).Close;
end;

constructor TCalendar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TabStop := True;
  FCreated := false;
  Constraints.MinWidth := 0;
  Constraints.MinHeight := 0;
  FMinHeaderWidth := 0;
  FMinBodyWidth := 0;
  FMinHeaderHeight := 0;
  FMinBodyHeight := 0;
  FMinTodayLabelHeight := 0;
  FEnabled := True;
  Self.BorderStyle := bsRaisedPanel;
  FDate := DateOf(Now);
  //Create & Setup Header
  FHeaderPanel := TPanel.Create(Self);
  with FHeaderPanel do
   begin
    Parent := Self;
    Align := alTop;
    BorderStyle := bsNone;
    BevelOuter := bvNone;
    BevelInner := bvNone;
    Color := $00A70000;
    FHeaderColor := Color;
    OnMouseMove := HeaderOnMouseMove;
   end;
  FMonth := TSpeedButton.Create(FHeaderPanel);
  with FMonth do
   begin
    Parent := FHeaderPanel;
    Flat := True;
    Caption := MonthName(FDate);
    Font.Color := clWhite;
    Font.Style := [fsBold];
    FoldMonthFontChangeProc := Font.OnChange;
    Font.OnChange := MonthFontChange;
    OnClick := ShowMonthMenu;
    OnMouseMove := MonthOnMouseMove;
   end;
  FYear := TEdit.Create(FHeaderPanel);
  with FYear do
   begin
    Parent := FHeaderPanel;
    Text := IntToStr(YearOf(fDate));
    FoldYearFontChangeProc := Font.OnChange;
    Font.OnChange := YearFontChange;
    Font.Color := clWhite;
    Color := FHeaderPanel.Color;
    BorderStyle := bsNone;
    Font.Style := [fsBold];
    OnExit := YearOnExit;
    OnEnter := YearOnEnter;
    OnKeyUp := YearOnKeyUp;
    OnMouseMove := YearOnMouseMove;
    Alignment := taCenter;
    TabStop := False;
    MaxLength := 4;
    FoldYear := Text;
   end;
  HeaderAutoSizeAndPlace;
  //Create & Setup Grid (Body)
  FBodyPanel := TPanel.Create(Self);
  with FBodyPanel do
   begin
    Parent := Self;
    Align := alClient;
    BorderStyle := bsNone;
    BevelOuter := bvNone;
    BevelInner := bvNone;
    Color := clWhite;
    OnMouseMove := BodyOnMouseMove;
   end;
  FillMonthGrid;
  FGrid := TDrawGrid.Create(FBodyPanel);
  with FGrid do
   begin
    Parent := FBodyPanel;
    Color := FBodyPanel.Color;
    Options := [];
    FixedCols := 0;
    FixedRows := 0;
    ScrollBars := ssNone;
    BorderStyle := bsNone;
    DefaultDrawing := False;
    OnDrawCell := CalendarDrawCell;
    OnClick := CalendarClick;
    OnMouseMove := GridOnMouseMove;
    RowCount := 7;
    ColCount := 7;
   end;
  FDayNamesFont := TFont.Create;
  with FDayNamesFont do
   begin
    Style := [fsBold];
    Color := clBlack;
   end;
  FTodayFont := TFont.Create;
  with FTodayFont do
   begin
    Style := [];
    Color := clBlue;
   end;
  FMonthDayFont := TFont.Create;
  with FMonthDayFont do
   begin
    Style := [];
    Color := clBlack;
   end;
  FOtherDayFont := TFont.Create;
  with FOtherDayFont do
   begin
    Size := FMonthDayFont.Size - 2;
    Style := [];
    Color := clSilver;
   end;
  AutoSizeCalendar;
  //Create & Setup Today Label (Footer)
  FTodayLabel := TLabel.Create(Self);
  with FTodayLabel do
   begin
    Parent := Self;
    fTodayLabelCaption := 'Today:';
    Caption := ' ' + FTodayLabelCaption + ' ' + DateToStr(fDate) + ' ';
    Align := alBottom;
    Alignment := taCenter;
    Font.Color := clBlack;
    Font.Style := [fsBold];
    OnMouseMove := TodayLabelOnMouseMove;
    Color := clWhite;
   end;
  FCreated := true;
end;

procedure TCalendar.DateChanged;
begin
  FYear.Text := IntToStr(YearOf(FDate));
  FMonth.Caption := MonthName(FDate);
  FillMonthGrid;
  FGrid.Invalidate;
  if Assigned(FOnDateChanged) then FOnDateChanged(Self);
end;

destructor TCalendar.Destroy;
begin
  FreeAndNil(FTodayLabel);
  FreeAndNil(FOtherDayFont);
  FreeAndNil(FMonthDayFont);
  FreeAndNil(FTodayFont);
  FreeAndNil(FDayNamesFont);
  FreeAndNil(FBodyPanel);
  FreeAndNil(FYear);
  FreeAndNil(FMonth);
  FreeAndNil(FHeaderPanel);
  inherited;
end;

procedure TCalendar.FillMonthGrid;
var
  CurDate: TDateTime;
  i,j: Byte;
  FirstWeekDay: Byte;
begin
  CurDate := EncodeDate(YearOf(FDate),MonthOf(FDate),1);
  FirstWeekDay := 1;
  case FFDow of
   dowMonday: FirstWeekDay := 1;
   dowTuesday: FirstWeekDay := 2;
   dowWednesday: FirstWeekDay := 3;
   dowThursday: FirstWeekDay := 4;
   dowFriday: FirstWeekDay := 5;
   dowSaturday: FirstWeekDay := 6;
   dowSunday: FirstWeekDay := 7;
  end;
  while DayOfTheWeek(CurDate) <> FirstWeekDay do CurDate := IncDay(CurDate,-1);
  for j := 1 to 6 do
   for i := 1 to 7 do
    begin
     FMonthGrid[i,j].Day := CurDate;
     FMonthGrid[i,j].Selected := False;
     FMonthGrid[i,j].DType := dtNormal;
     if isSameDay(CurDate,Now) then FMonthGrid[i,j].DType := dtCurrent;
     if isSameDay(CurDate,FDate) then FMonthGrid[i,j].Selected := True;
     if MonthOf(CurDate) <> MonthOf(FDate) then
      begin
       if CurDate < FDate then FMonthGrid[i,j].DType := dtPrevMonth
                          else FMonthGrid[i,j].DType := dtNextMonth;
      end;
     CurDate := IncDay(CurDate,1);
    end;
end;

procedure TCalendar.GridOnMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  If Assigned(FOnMouseMove) then
   begin
    FOnMouseMove(Sender,Shift,FGrid.Left+X,
                 FBodyPanel.Top+FGrid.Top+Y);
   end;
end;

procedure TCalendar.HeaderAutoSizeAndPlace;
const
  MinSpaceX = 10;
  TopPos = 7;
var
  i :byte;
  temp :integer;
begin
  //Fix vertical position and Header height
  FMonth.Top := TopPos;
  FYear.Top := TopPos;
  temp := Max(FMonth.Height,FYear.Height);
  FHeaderPanel.Height := temp+TopPos*2;
  //Fix horizontal positions and widths
  temp := 0;
  for i := 1 to 12
   do temp := Max(temp,TextWidth(FMonth.Font,LongMonthNames[i]));
  FMonth.Width := temp+10;
  FYear.Width := TextWidth(fYear.Font,'0000')+10;
  FMinHeaderWidth := FMonth.Width + FYear.Width + MinSpaceX*3;
  temp := Width - (FMonth.Width + FYear.Width);
  temp := temp div 3;
  FMonth.Left := temp;
  FYear.Left := FMonth.Left + FMonth.Width + temp;
  FMonth.Height := max(FMonth.Height,FYear.Height);
  FYear.Height := FMonth.Height;
  FMinHeaderHeight := FHeaderPanel.Height;
end;

procedure TCalendar.HeaderOnMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  If Assigned(FOnMouseMove) then
   begin
    FOnMouseMove(Sender,Shift,FHeaderPanel.Left+X,FHeaderPanel.Top+Y);
   end;
end;

procedure TCalendar.MonthButtonClick(Sender: TObject);
var
 MyObj: TObject;
 MyYear,MyMonth,MyDay: Word;
 MyIntMonth: Integer;
begin
  DecodeDate(FDate,MyYear,MyMonth,MyDay);
  MyIntMonth := StrToInt(Copy((Sender as TSpeedButton).Name,3,
                              length((Sender as TSpeedButton).Name)-2));
  MyObj := (Sender as TSpeedButton).Parent.Parent;
  (MyObj as TForm).Close;
  if DateOf(FDate) <> EncodeDate(MyYear,MyIntMonth,MyDay) then
   begin
    FDate := EncodeDate(MyYear,MyIntMonth,MyDay);
    DateChanged;
   end; 
end;

procedure TCalendar.MonthFontChange(Sender: TObject);
begin
  FoldMonthFontChangeProc(Sender);
  AutoSizeAndPlace;
end;

function TCalendar.MonthName(Dt: TDateTime): String;
begin
  Result := LongMonthNames[MonthOf(Dt)];
  ReSult := UpperCase(Copy(Result,1,1))+Copy(Result,2,Length(Result)-1);
end;

procedure TCalendar.MonthOnMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  If Assigned(FOnMouseMove) then
   begin
    FOnMouseMove(Sender,Shift,FMonth.Left+X,FMonth.Top+Y);
   end;
end;

function TCalendar.ReadCalColor: TColor;
begin
  Result := FBodyPanel.Color;
end;

function TCalendar.ReadDayNamesFont: TFont;
begin
  Result := FDayNamesFont;
end;

function TCalendar.ReadHeaderColor: TColor;
begin
  Result := FHeaderColor;
end;

function TCalendar.ReadLabelAlignment: TAlignment;
begin
  Result := FTodayLabel.Alignment;
end;

function TCalendar.ReadLabelCaption: string;
begin
  Result := FTodayLabelCaption;
end;

function TCalendar.ReadLabelColor: TColor;
begin
  Result := FTodayLabel.Color;
end;

function TCalendar.ReadLabelFont: TFont;
begin
  Result := FTodayLabel.Font;
end;

function TCalendar.ReadMonthDayFont: TFont;
begin
  Result := FMonthDayFont;
end;

function TCalendar.ReadMonthFont: TFont;
begin
  Result := FMonth.Font;
end;

function TCalendar.ReadOtherDayFont: TFont;
begin
  Result := FOtherDayFont;
end;

function TCalendar.ReadTodayFont: TFont;
begin
  Result := FTodayFont;
end;

function TCalendar.ReadYearFont: TFont;
begin
  Result := FYear.Font;
end;

procedure TCalendar.Repaint;
begin
  inherited;
  AutoSizeAndPlace;
  FHeaderPanel.Repaint;
  FBodyPanel.Repaint;
  FTodayLabel.Repaint;
  FYear.Repaint;
  FMonth.Repaint;
  FGrid.Repaint;
end;

procedure TCalendar.Resize;
begin
  inherited;
  if FCreated then AutoSizeAndPlace;
end;

procedure TCalendar.SetCalColor(const Value: TColor);
begin
  FBodyPanel.Color := Value;
  FGrid.Color := Value;
  FBodyPanel.Invalidate;
  FGrid.Invalidate;
end;

procedure TCalendar.SetDate(const Value: TDateTime);
begin
  if DateOf(FDate) <> DateOf(Value) then
   begin
    FDate := DateOf(Value);
    DateChanged;
   end;
end;

procedure TCalendar.SetDayNamesFont(const Value: TFont);
begin
  FDayNamesFont.Assign(Value);
  FGrid.Invalidate;
end;

procedure TCalendar.SetEnabled(const Value: Boolean);
begin
  inherited;
  if FEnabled <> Value then
   begin
    FEnabled := Value;
    FGrid.Enabled := Value;
    FYear.Enabled := Value;
    FMonth.Enabled := Value;
    FTodayLabel.Enabled := Value;
    FHeaderPanel.Enabled := Value;
    FBodyPanel.Enabled := Value;
    if FEnabled then FHeaderPanel.Color := FHeaderColor
                else FHeaderPanel.Color := clDisabledBackGround;
   end;
end;

procedure TCalendar.SetFDoW(const Value: TDoW);
begin
  FFDoW := Value;
end;

procedure TCalendar.SetHeaderColor(const Value: TColor);
begin
  FHeaderColor := Value;
  FHeaderPanel.Color := Value;
  FYear.Color := Value;
end;

procedure TCalendar.SetLabelAlignment(const Value: TAlignment);
begin
  FTodayLabel.Alignment := Value;
end;

procedure TCalendar.SetLabelCaption(const Value: string);
begin
  FTodayLabelCaption := Value;
  FTodayLabel.Caption := ' ' + FTodayLabelCaption + ' ' + DateToStr(Now) + ' ';
end;

procedure TCalendar.SetLabelColor(const Value: TColor);
begin
  FTodayLabel.Color := Value;
end;

procedure TCalendar.SetLabelFont(const Value: TFont);
begin
  FTodayLabel.Font.Assign(Value);
end;

procedure TCalendar.SetMonthDayFont(const Value: TFont);
begin
  FMonthDayFont.Assign(Value);
  FGrid.Invalidate;
end;

procedure TCalendar.SetMonthFont(const Value: TFont);
begin
  FMonth.Font.Assign(Value);
end;

procedure TCalendar.SetOtherDayFont(const Value: TFont);
begin
  FOtherDayFont.Assign(Value);
  FGrid.Invalidate;
end;

procedure TCalendar.SetTodayFont(const Value: TFont);
begin
  FTodayFont.Assign(Value);
  FGrid.Invalidate;
end;

procedure TCalendar.SetYearFont(const Value: TFont);
begin
  FYear.Font.Assign(Value);
end;

procedure TCalendar.ShowMonthMenu(Sender: TObject);
var
 MyTempForm: TForm;
 i: Byte;
 MyTop: Integer;
 MyPoint: TPoint;
 MyFrame: TPanel;
begin
  MyTempForm := TForm.Create(Application);
  try
   with MyTempForm do
    begin
     Parent := Self;
     BorderStyle := fbsNone;
     MyPoint.X := FMonth.Left;
     MyPoint.Y := FMonth.Top + FMonth.Height;
     Left := Parent.ClientToScreen(MyPoint).X;
     Top := Parent.ClientToScreen(MyPoint).Y+1;
     MyTop := 0;
     OnDeactivate := CloseMonthMenu;
     MyFrame := TPanel.Create(MyTempForm);
     with MyFrame do
      begin
       Parent := MyTempForm;
       Align := alClient;
       Color := clWhite;
       BorderStyle := bsNone;
       BevelOuter := bvLowered;
       BevelInner := bvNone;
      end;
     Width := FMonth.Width*2+2;
     for i := 1 to 12 do
      begin
       with TSpeedButton.Create(MyFrame) do
        begin
         Font.Assign(FMonth.Font);
         Caption := UpperCase(Copy(LongMonthNames[i],1,1))+
                    Copy(LongMonthNames[i],2,Length(LongMonthNames[i])-1);
         Font.Color := clBlack;
         Name := 'BT'+IntToStr(i);
         Parent := MyFrame;
         Flat := True;
         Height := FMonth.Height;
         Width := FMonth.Width;
         OnClick := MonthButtonClick;
         if Odd(i) then
          begin
           Left := 1;
           Top := MyTop;
          end else
          begin
           Left := Width+1;
           Top := MyTop;
           MyTop := MyTop+Height;
          end;
        end;
      end;
     Height := MyTop;
     ShowModal;
    end;
  finally
   for i := 1 to 12 do MyTempForm.FindComponent('BT'+IntToStr(i)).Free;
   FreeAndNil(MyFrame);
   FreeAndNil(MyTempForm);
  end;
end;

function TCalendar.TextHeight(Font: TFont; Text: String): integer;
var
  MyBitmap :TBitmap;
begin
  MyBitmap := TBitmap.Create; //We need a temporary Canvas
                              //if you need speed you should create this once
                              //for all in the component's Create procedure
  MyBitmap.Width := 1;
  MyBitmap.Height := 1;
  MyBitmap.Canvas.Font := Font;
  Result := MyBitmap.Canvas.TextHeight(Text);
  FreeAndNil(MyBitmap);
end;

function TCalendar.TextWidth(Font: TFont; Text: String): integer;
var
  MyBitmap :TBitmap;
begin
  MyBitmap := TBitmap.Create; //We need a temporary Canvas
                              //if you need speed you should create this once
                              //for all in the component's Create procedure
  MyBitmap.Width := 1;
  MyBitmap.Height := 1;
  MyBitmap.Canvas.Font := Font;
  Result := MyBitmap.Canvas.TextWidth(Text);
  FreeAndNil(MyBitmap);
end;

procedure TCalendar.TodayLabelOnMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  If Assigned(FOnMouseMove) then
   begin
    FOnMouseMove(Sender,Shift,FTodayLabel.Left+X,FTodayLabel.Top+Y);
   end;
end;

procedure TCalendar.YearFontChange(Sender: TObject);
begin
  FoldYearFontChangeProc(Sender);
  if FCreated then AutoSizeAndPlace;
end;

procedure TCalendar.YearOnEnter(Sender: TObject);
begin
  FoldYear := FYear.Text;
  FYear.Color := clWhite;
  FoldYearColor := FYear.Font.Color;
  FYear.Font.Color := clBlack;
  FYear.BorderStyle := bsSingle;
end;

procedure TCalendar.YearOnExit(Sender: TObject);
var
  MyYear, MyMonth, MyDay: Word;
  MyIntYear: Integer;
begin
  FYear.Color := FHeaderPanel.Color;
  FYear.BorderStyle := bsNone;
  FYear.Font.Color := FoldYearColor;
  if length(FYear.Text) <> 4 then FYear.Text := FoldYear;
  if TryStrToInt(FYear.Text,MyIntYear) then
   begin
    if (MyIntYear >= 1000) AND (MyIntYear <= 3000) AND (FYear.Text <> FoldYear)
     then begin
      DecodeDate(FDate,MyYear,MyMonth,MyDay);
      FDate := EncodeDate(MyIntYear,MyMonth,MyDay);
      DateChanged;
     end else FYear.Text := FoldYear;
   end else FYear.Text := FoldYear;
end;

procedure TCalendar.YearOnKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = Key_Return then FGrid.SetFocus;
  if Key = Key_Escape then FYear.Text := FoldYear;
end;

procedure TCalendar.YearOnMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  If Assigned(FOnMouseMove) then
   begin
    FOnMouseMove(Sender,Shift,FYear.Left+X,FYear.Top+Y);
   end;
end;

end.
